from peacemakr.crypto_base import PeacemakrCryptoSDK

from peacemakr.impl.models.cipher_text_aad import CiphertextAAD

from peacemakr.generated.api_client import ApiClient
from peacemakr.generated.api.client_api import ClientApi
from peacemakr.generated.api.crypto_config_api import CryptoConfigApi
from peacemakr.generated.api.org_api import OrgApi
from peacemakr.generated.api.key_service_api import KeyServiceApi

from peacemakr.generated.models.client import Client
from peacemakr.generated.configuration import Configuration
from peacemakr.generated.models import *
from peacemakr.generated.rest import ApiException

from peacemakr.exception.core_crypto import CoreCryptoError
from peacemakr.exception.failed_to_download_key import FailedToDownloadKeyError
from peacemakr.exception.invalid_cipher import InvalidCipherError
from peacemakr.exception.missing_api_key import MissingAPIKeyError
from peacemakr.exception.missing_client_name import MissingClientNameError
from peacemakr.exception.missing_persister import MissingPersisterError
from peacemakr.exception.no_valid_use_domains_for_encryption import NoValidUseDomainsForEncryptionError
from peacemakr.exception.no_valid_use_domains_for_decryption import NoValidUseDomainsForDecryptionError
from peacemakr.exception.peacemakr import PeacemakrError
from peacemakr.exception.persistence_layer_corruption_detected import PersistenceLayerCorruptionDetectedError
from peacemakr.exception.server import ServerError
from peacemakr.exception.unrecoverable_clock_skew_detected import UnrecoverableClockSkewDetectedError

from peacemakr.crypto_base import PeacemakrCryptoSDK
from peacemakr.impl.persister_impl import InMemoryPersister
from peacemakr.persister_base import Persister
import peacemakr_core_crypto_python as p

from random import randint
from functools import reduce
import logging
import time
import json
import base64
import sys

PERSISTER_PRIV_KEY = "Priv"
PERSISTER_PUB_KEY = "Pub"
PERSISTER_ASYM_TYPE = "AsymmetricKeyType"
PERSISTER_ASYM_CREATED_DATE_EPOCH = "AsymmetricKeyCreated"
PERSISTER_ASYM_BITLEN = "AsymmetricKeyBitlen"
PERSISTER_CLIENTID_KEY = "ClientId"
PERSISTER_PREFERRED_KEYID = "PreferredKeyId"
PERSISTER_APIKEY_KEY = "ApiKey"

DEFAULT_SYMM_CIPHER = p.SymmetricCipher.CHACHA20_POLY1305
DEFAULT_MESSAGE_DIGEST = p.DigestAlgorithm.SHA_256
MAX_ELASPED_TIME = 60 * 60 * 24

def random_index(l: list):
    assert isinstance(l, list) and len(l)
    return l[randint(0, len(l) - 1)]

class CryptoImpl(PeacemakrCryptoSDK):
    def __init__(self, api_key="", client_name="", peacemakr_hostname="", persister=None, logger=None):
        assert isinstance(api_key, str)
        assert isinstance(client_name, str)
        assert isinstance(peacemakr_hostname, str)
        assert logger is None or isinstance(logger, logging.Logger)
        if not client_name:
            raise MissingClientNameError("A client name is required")
        if not api_key:
            raise MissingAPIKeyError("An API key is required")
        if not persister or not isinstance(persister, Persister):
            raise MissingPersisterError("A persister is required")

        self.api_key = api_key
        self.client_name = client_name
        self.sdk_version = "0.0.1"
        self.peacemakr_hostname = peacemakr_hostname
        self.org = None
        self.crypto_config = None
        self.persister = persister
        self.logger = logger or logging.getLogger("default_logger")
        logging.basicConfig(level=logging.WARNING, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        self.__client = None
        self.__api_client = None
        self.__authentication = None
        self.__loaded_private_preferred_key = None
        self.__loaded_private_preferred_cipher = None
        self.__crypto_context = p.CryptoContext()

        self.__last_updated_time = None
        self.__sym_key_cache = dict()

    def __bootsrapped_private_preferred_key_and_cipher(self):
        self.__loaded_private_preferred_cipher = self.__get_asymmetric_cipher(self.persister.load(PERSISTER_ASYM_TYPE), self.persister.load(PERSISTER_ASYM_BITLEN))
        self.__loaded_private_preferred_key = p.Key(DEFAULT_SYMM_CIPHER, self.persister.load(PERSISTER_PRIV_KEY), True)

    def __is_registered(self) -> bool:
        return  self.persister.exists(PERSISTER_PREFERRED_KEYID)\
            and self.persister.exists(PERSISTER_CLIENTID_KEY)\
            and self.persister.exists(PERSISTER_PRIV_KEY)\
            and self.persister.exists(PERSISTER_PUB_KEY)\
            and self.persister.exists(PERSISTER_ASYM_TYPE)\
            and self.__loaded_private_preferred_key != None\
            and self.__loaded_private_preferred_cipher != None

    def __is_bootstrapped(self) -> bool:
        return self.org != None and self.crypto_config != None and self.__client != None

    def __update_config_by_elasped_time(self, max_elasped_time: int = 60*60*24):
        ''' update the config if program elasped more than `max_elasped_time` since the last update
        '''
        now = time.time()
        if (now - self.__last_updated_time) > max_elasped_time:
            # add logger
            self.logger.info("Process elasped more than {} secs, updating config".format(max_elasped_time))
            self.sync()
            self.__last_updated_time = time.time()

    def __do_bootstrap_org_and_crypto_config(self):
        # set up org, api_client, and crypto_config
        if self.__is_bootstrapped():
            self.logger.info("client is bootstrapped already")
            return

        api_client = self.__get_client()

        self.__load_org(api_client)
        self.__load_crypto_config(api_client)

    def __load_org(self, api_client: ApiClient):
        assert isinstance(api_client, ApiClient)
        org_api = OrgApi(api_client=api_client)
        try:
            self.org = org_api.get_organization_from_api_key(apikey=self.api_key)
        except ApiException as e:
            self.logger.warning("Excpetion in getting organization from api key: {}".format(e))
            raise ServerError(e)

    def __load_crypto_config(self, api_client: ApiClient):
        assert isinstance(api_client, ApiClient)
        crypto_config_api = CryptoConfigApi(api_client=api_client)
        try:
            self.crypto_config = crypto_config_api.get_crypto_config(self.org.crypto_config_id)
        except ApiException as e:
            self.logger.warning("Exception in getting crypto config: {}".format(e))
            raise ServerError(e)



    def __get_client(self) -> ApiClient:
        if self.__api_client != None:
            return self.__api_client

        if self.api_key == "":
            self.logger.warning("Missing API Key")
            raise MissingAPIKeyError("Missing API Key")

        configuration = Configuration()
        configuration.api_key['authorization'] = self.api_key
        configuration.host = self.peacemakr_hostname + "/api/v1"
        self.__api_client = ApiClient(configuration=configuration)

        self.persister.save(PERSISTER_APIKEY_KEY, self.api_key)
        return self.__api_client

    def __gen_new_asymmetric_keypair(self, persister: Persister) -> PublicKey:
        assert isinstance(persister, Persister)
        rand = p.RandomDevice()

        symm_cipher = DEFAULT_SYMM_CIPHER
        asymm_cipher = self.__get_asymmetric_cipher(self.crypto_config.client_key_type, self.crypto_config.client_key_bitlength)

        key = p.Key(asymm_cipher, symm_cipher, rand)

        pub_pem = key.get_pub_pem()
        priv_pem = key.get_priv_pem()

        created_time = int(round(time.time()))
        if created_time > sys.maxsize:
            self.logger.error("Failed to detect a valid time for local asymmetric key creation time")
            raise UnrecoverableClockSkewDetectedError("Failed to detect a valid time for local asymmetric key creation time")

        pub_key = PublicKey(id="",
                            key=pub_pem,
                            creation_time=created_time,
                            key_type=self.crypto_config.client_key_type,
                            encoding="pem")

        persister.save(PERSISTER_PRIV_KEY, priv_pem)
        persister.save(PERSISTER_PUB_KEY, pub_pem)
        persister.save(PERSISTER_ASYM_TYPE, self.crypto_config.client_key_type)
        persister.save(PERSISTER_ASYM_CREATED_DATE_EPOCH, created_time)
        persister.save(PERSISTER_ASYM_BITLEN, self.crypto_config.client_key_bitlength)

        return pub_key

    def __get_asymmetric_cipher(self, key_type: str, bit_length: int) -> p.AsymmetricCipher:
        assert isinstance(key_type, str)
        assert isinstance(bit_length, int)

        prefix_dict = {
            "ec" : "ECDH_P",
            "rsa": "RSA_"
        }
        asymm_dict = {
            "RSA_2048": p.AsymmetricCipher.RSA_2048,
            "RSA_4096": p.AsymmetricCipher.RSA_4096,
            "ECDH_P256": p.AsymmetricCipher.ECDH_P256,
            "ECDH_P384": p.AsymmetricCipher.ECDH_P384,
            "ECDH_P521": p.AsymmetricCipher.ECDH_P521,
        }
        prefix = prefix_dict[key_type.lower()]
        asymm_key = '{}{}'.format(prefix, bit_length)

        if asymm_key not in asymm_dict:
            self.logger.warning('{} is not currently supported by Peacemakr. Going to default ECDH_P521'.format(asymm_key))
            return p.AsymmetricCipher.ECDH_P521

        return asymm_dict[asymm_key]

    def __get_symmetric_cipher(self, symmetric_alg: str) -> p.SymmetricCipher:
        assert isinstance(symmetric_alg, str)
        select_cipher = {
            "CHACHA20_POLY1305": p.SymmetricCipher.CHACHA20_POLY1305,
            "AES-128-GCM": p.SymmetricCipher.AES_128_GCM,
            "AES-192-GCM": p.SymmetricCipher.AES_192_GCM,
            "AES-256-GCM": p.SymmetricCipher.AES_256_GCM,
        }
        if symmetric_alg not in select_cipher:
            self.logger.warning('{} is not currently supported by Peacemakr. Going to default {}'.format(symmetric_alg, DEFAULT_SYMM_CIPHER))
            return DEFAULT_SYMM_CIPHER

        return select_cipher[symmetric_alg]

    def __get_digest_alg(self, digest_algorithm: str) -> p.DigestAlgorithm:
        assert isinstance(digest_algorithm, str)
        select_digest = {
            "SHA_224" : p.DigestAlgorithm.SHA_224,
            "SHA_256" : p.DigestAlgorithm.SHA_256,
            "SHA_384" : p.DigestAlgorithm.SHA_384,
            "SHA_512" : p.DigestAlgorithm.SHA_512,
        }
        if digest_algorithm not in select_digest:
            self.logger.warning('{} is not currently supported by Peacemakr. Going to default {}'.format(digest_algorithm, DEFAULT_MESSAGE_DIGEST))
            return DEFAULT_MESSAGE_DIGEST

        return select_digest[digest_algorithm]

    def __verify_bootstrapped_and_registered(self):
        if not self.__is_registered() or not self.__is_bootstrapped():
            self.logger.warning("SDK was not registered, please register before using other SDK operations")
            raise PeacemakrError("SDK was not registered, please register before using other SDK operations")

    def __save_new_asymmetric_key_pair(self, src: Persister, dst: Persister):
        assert isinstance(src, Persister)
        assert isinstance(dst, Persister)
        dst.save(PERSISTER_PRIV_KEY, src.load(PERSISTER_PRIV_KEY))
        dst.save(PERSISTER_PUB_KEY, src.load(PERSISTER_PUB_KEY))
        dst.save(PERSISTER_ASYM_TYPE, src.load(PERSISTER_ASYM_TYPE))
        dst.save(PERSISTER_ASYM_CREATED_DATE_EPOCH, src.load(PERSISTER_ASYM_CREATED_DATE_EPOCH))
        dst.save(PERSISTER_ASYM_BITLEN, src.load(PERSISTER_ASYM_BITLEN))

    def __gen_and_register_new_preferred_client_key(self):
        temp_in_memory_persister = InMemoryPersister()
        public_key = self.__gen_new_asymmetric_keypair(temp_in_memory_persister)

        client_api = ClientApi(self.__get_client())
        try:
            public_key = client_api.add_client_public_key(self.__client.id, public_key)
        except ApiException as e:
            self.logger.warning("Exception in adding client public key: {}".format(e))
            raise ServerError(e)

        self.__save_new_asymmetric_key_pair(temp_in_memory_persister, self.persister)
        self.persister.save(PERSISTER_PREFERRED_KEYID, public_key.id)


    def __update_local_crypto_config(self, new_config: p.CryptoConfig):
        assert isinstance(new_config, p.CryptoConfig)
        cur_asymmetric_key_type = self.persister.load(PERSISTER_ASYM_TYPE)
        if cur_asymmetric_key_type != new_config.client_key_type:
            self.logger.info("update client asymmetric key type and crypto config from {} to {}".format(cur_asymmetric_key_type, new_config.client_key_type))
            self.crypto_config = new_config
            self.__gen_and_register_new_preferred_client_key()
            return

        cur_asymmetric_key_creation_time = self.persister.load(PERSISTER_ASYM_CREATED_DATE_EPOCH)
        asymmetric_key_creation_time = cur_asymmetric_key_creation_time
        if (new_config.client_key_ttl + asymmetric_key_creation_time) > int(round(time.time())):
            self.logger.info("client key expired, generating new client key")
            self.crypto_config = new_config
            self.__gen_and_register_new_preferred_client_key()
            return

        cur_asymmetric_key_bit_lens = self.persister.load(PERSISTER_ASYM_BITLEN)
        asymmertric_key_bit_len = int(cur_asymmetric_key_bit_lens)
        if asymmertric_key_bit_len != new_config.client_key_bitlength:
            self.logger.info("update client key length and crypto config from {} to {}".format(asymmertric_key_bit_len, new_config.client_key_bitlength))
            self.crypto_config = new_config
            self.__gen_and_register_new_preferred_client_key()
            return

        self.crypto_config = new_config

    def __download_and_save_all_keys(self, required_keys: list =[]):
        assert isinstance(required_keys, list)
        ''' calls keyServiceApi to get all the encrypted keys just generated from the key deriver
        '''
        key_api = KeyServiceApi(api_client=self.__get_client())
        try:
            all_keys = key_api.get_all_encrypted_keys(self.__client.preferred_public_key_id, symmetric_key_ids=required_keys)            
        except ApiException as e:
            self.logger.warning("Exception in getting all encrypted keys: {}".format(e))
            raise ServerError(e)

        self.__decrypt_and_save(all_keys)



    def __decrypt_and_save(self, all_keys: list):
        ''' decryptes the encrypted symmetric keys and save them to persister
        '''
        assert isinstance(all_keys, list)
        # loaded private preferred key should be loaded at boostrap
        if self.__loaded_private_preferred_key == None:
            self.logger.warning("SDK was not registered, please register before using other SDK operations")
            raise PeacemakrError("SDK was not registered, please register before using other SDK operations")

        for key in all_keys:
            if key == None:
                continue

            raw_cipher_text_str = key.packaged_ciphertext
            if raw_cipher_text_str == None:
                self.logger.info("Failed to get raw ciphertext str from EncryptedSymmetricKey")
                continue

            # deserialized[0] is a pycapsule object that decrypt takes in,
            # deseralized[1] is the config (sym cipher, asym cipher, digest algo, and mode)
            deserialized = self.__crypto_context.deserialize(raw_cipher_text_str)

            extracted_aad = self.__crypto_context.extract_unverified_aad(raw_cipher_text_str)
            # check if extracted aad is null
            if extracted_aad == "" or not extracted_aad:
                self.logger.warning("Failed to extrat aad from the ciphertext")
                raise CoreCryptoError("Failed to extract aad from the ciphertext")

            aad = json.loads(extracted_aad.aad)

            verification_key = self.__get_or_download_public_key(aad["senderKeyID"])

            if self.__is_ec(self.__loaded_private_preferred_cipher):
                # verfication is the peer key and client key is "my_key"
                if isinstance(verification_key, str):
                    verification_key = p.Key(DEFAULT_SYMM_CIPHER, verification_key, False)

                shared_symm_key = p.Key(DEFAULT_SYMM_CIPHER, self.__loaded_private_preferred_key, verification_key)
                result = self.__crypto_context.decrypt(shared_symm_key, deserialized[0])
            elif self.__is_rsa(self.__loaded_private_preferred_cipher):
                result = self.__crypto_context.decrypt(self.__loaded_private_preferred_key, deserialized[0])
            else:
                self.logger.warning("This version of python SDK only support ec or rsa. Invalid cipher.")
                raise InvalidCipherError("This version of python SDK only support ec or rsa. Invalid cipher.")

            # data is the plaintext, result = [plainText, NeedVerify:bool]; plainText = {data, aad}
            need_verfication = result[1]
            # check verfication
            if need_verfication:
                verified = self.__crypto_context.verify(verification_key, result[0], deserialized[0])
                if verified == False:
                    raise CoreCryptoError('Verification step failed')

            keys_in_str = result[0].data

            key_len = key.key_length
            keys_in_bytes = base64.decodebytes(keys_in_str)
            for i in range(len(key.key_ids)):
                # loop thru each key id and save their respective keys
                key_in_bytes = keys_in_bytes[i*key_len:(i+1)*key_len]
                self.__sym_key_cache[key.key_ids[i]] = key_in_bytes
                self.persister.save(key.key_ids[i], key_in_bytes)

    def __is_ec(self, cipher: p.AsymmetricCipher) -> bool:
        assert isinstance(cipher, p.AsymmetricCipher)
        if cipher in {p.AsymmetricCipher.ECDH_P256, p.AsymmetricCipher.ECDH_P384, p.AsymmetricCipher.ECDH_P521}:
            return True
        return False

    def __is_rsa(self, cipher: p.AsymmetricCipher) -> bool:
        assert isinstance(cipher, p.AsymmetricCipher)
        if cipher in {p.AsymmetricCipher.RSA_2048, p.AsymmetricCipher.RSA_4096}:
            return True
        return False

    def __get_or_download_public_key(self, key_id: str) -> p.Key:
        assert isinstance(key_id, str)
        #FIXME: should we really use default symm cipher all the time?
        if (self.persister.exists(key_id)):
            return p.Key(DEFAULT_SYMM_CIPHER, self.persister.load(key_id), False)

        key_service_api = KeyServiceApi(self.__get_client())

        try:
            pub_key = key_service_api.get_public_key(key_id)
        except ApiException as e:
            self.logger.warning("Exception in getting public key: {}".format(e))
            raise ServerError(e)

        self.persister.save(key_id, pub_key.key)

        return p.Key(DEFAULT_SYMM_CIPHER, pub_key.key, False)


    def register(self):
        # check is register and is boostrap, if not initialize
        if self.__is_registered():
            if not self.__is_bootstrapped():
                self.__do_bootstrap_org_and_crypto_config()
            self.logger.info("User is registered and boostrapped already")
            return

        self.__do_bootstrap_org_and_crypto_config()
        # generate new asymmetric client keypair and then store info in persistor
        pub_key = self.__gen_new_asymmetric_keypair(self.persister)

        # generate new client with empty id and client public key
        client = Client(id = "", public_keys=[pub_key], sdk=self.sdk_version)

        client_api = ClientApi(api_client=self.__get_client())

        try:
            new_client = client_api.add_client(client)
        except ApiException as e:
            self.logger.warning("Exception in getting all adding client: {}".format(e))
            raise ServerError(e)

        self.__client = new_client

        # FIXME: add client checks like in Java?
        self.__bootsrapped_private_preferred_key_and_cipher()
        self.persister.save(PERSISTER_CLIENTID_KEY, self.__client.id)
        self.persister.save(PERSISTER_PREFERRED_KEYID, self.__client.public_keys[0].id)
        self.__last_updated_time = time.time()


    def sync(self):
        self.__verify_bootstrapped_and_registered()
        crypto_config_api = CryptoConfigApi(self.__get_client())

        try:
            new_config = crypto_config_api.get_crypto_config(self.crypto_config.id)
            if new_config != self.crypto_config:
                self.__update_local_crypto_config(new_config)
        except ApiException as e:
            self.logger.warning("Exception in getting crypto config: {}".format(e))
            raise ServerError(e)

        self.__download_and_save_all_keys()

    def __domain_is_valid_for_encryption(self, domain: SymmetricKeyUseDomain) -> bool:
        assert isinstance(domain, SymmetricKeyUseDomain)
        now_in_seconds = int(round(time.time()))
        return domain.creation_time + domain.symmetric_key_encryption_use_ttl > now_in_seconds \
               and domain.creation_time + domain.symmetric_key_inception_ttl <= now_in_seconds

    def __select_use_domain_name(self) -> str:
        domains = self.crypto_config.symmetric_key_use_domains
        valid_for_encryption = [d for d in domains if self.__domain_is_valid_for_encryption(d)]
        return random_index(valid_for_encryption) if valid_for_encryption else None

    def __get_valid_use_domain_for_encryption(self, use_domain_name: str) -> SymmetricKeyUseDomain:
        assert isinstance(use_domain_name, str)
        domains = self.crypto_config.symmetric_key_use_domains
        # TODO : Handle logger call.
        domains = list(filter(lambda d: d.name == use_domain_name, domains))
        domains = [d for d in domains if self.__domain_is_valid_for_encryption(d) and d.encryption_key_ids]
        if not domains:
            raise NoValidUseDomainsForEncryptionError("No valid use domain for encryption found, with the name " + use_domain_name)

        return random_index(domains)

    def __get_encryption_key_id(self, use_domain: SymmetricKeyUseDomain) -> str:
        assert isinstance(use_domain, SymmetricKeyUseDomain)
        return random_index(use_domain.encryption_key_ids)

    def __get_key(self, key_id: str) -> bytes:
        assert isinstance(key_id, str)

        if key_id in self.__sym_key_cache:
            return self.__sym_key_cache[key_id]

        if self.persister.exists(key_id):
            return self.persister.load(key_id)

        self.__download_and_save_all_keys([key_id])
        if not self.persister.exists(key_id):
            raise FailedToDownloadKeyError('KeyID: {}'.format(key_id))
        # TODO : Handle persister missing keys exception.

        return self.persister.load(key_id)



    def __get_signing_key(self, use_domain: SymmetricKeyUseDomain) -> p.Key:
        assert isinstance(use_domain, SymmetricKeyUseDomain)
        if use_domain.digest_algorithm is None:
            return None

        if self.__loaded_private_preferred_key == None:
            self.logger.warning("SDK was not registered, please register before using other SDK operations")
            raise PeacemakrError("SDK was not registered, please register before using other SDK operations")

        return self.__loaded_private_preferred_key



    def encrypt(self, plain_text: bytes) -> bytes:
        assert isinstance(plain_text, bytes)

        self.__verify_bootstrapped_and_registered()
        self.__update_config_by_elasped_time(MAX_ELASPED_TIME)
        used_domain_name = self.__select_use_domain_name()
        return self.encrypt_in_domain(plain_text, used_domain_name.name)

    def encrypt_in_domain(self, plain_text: bytes, use_domain_name: str) -> bytes:
        assert isinstance(plain_text, bytes)
        assert isinstance(use_domain_name, str)

        self.__verify_bootstrapped_and_registered()
        self.__update_config_by_elasped_time(MAX_ELASPED_TIME)
        use_domain_for_encryption = self.__get_valid_use_domain_for_encryption(use_domain_name)
        encryption_key_id_for_encryption = self.__get_encryption_key_id(use_domain_for_encryption)

        symmetric_cipher = self.__get_symmetric_cipher(use_domain_for_encryption.symmetric_key_encryption_alg)
        signing_key = self.__get_signing_key(use_domain_for_encryption)
        digest = self.__get_digest_alg(use_domain_for_encryption.digest_algorithm)

        key = p.Key(symmetric_cipher, self.__get_key(encryption_key_id_for_encryption))

        # FIXME persister preferred keyid can cause delay if persister read from disk
        aad = CiphertextAAD(encryption_key_id_for_encryption, self.persister.load(PERSISTER_PREFERRED_KEYID))
        json_bytes = bytes(json.dumps(aad.__dict__), encoding='utf8')

        pm_plain_text = p.Plaintext(plain_text, json_bytes)
        random_device = p.RandomDevice()

        cipher_text = self.__crypto_context.encrypt(key, pm_plain_text, random_device)

        if not self.__crypto_context.sign(signing_key, pm_plain_text, digest, cipher_text):
            self.logger.warning('Signing failed. Verification not required when decrypting.')

        return self.__crypto_context.serialize(digest, cipher_text).encode(encoding='UTF-8')


    def __parse_cipher_text_AAD(self, aad: bytes) -> CiphertextAAD:
        assert isinstance(aad, bytes)
        j = json.loads(aad)
        return CiphertextAAD(**j)

    def __verify_message(self, aad: CiphertextAAD, cfg: p.CryptoConfig, ciphertext, plaintext: p.Plaintext) -> bool:
        assert isinstance(aad, CiphertextAAD)
        assert isinstance(cfg, p.CryptoConfig)
        assert isinstance(plaintext, p.Plaintext)
        # FIXME: figure out import for this
        # assert isinstance(ciphertext, PyCapsule)

        sender_key = self.__get_or_download_public_key(aad.senderKeyID)
        return self.__crypto_context.verify(sender_key, plaintext, ciphertext)

    def __find_viable_decryption_use_domains(self) -> list:
        use_domains = self.crypto_config.symmetric_key_use_domains
        viable_domains = [domain for domain in use_domains if domain.symmetric_key_decryption_allowed]
        return viable_domains

    def __is_key_id_decryption_viable(self, key_id: str) -> bool:
        assert isinstance(key_id, str)
        viable_decryption_domains = self.__find_viable_decryption_use_domains()
        for domain in viable_decryption_domains:
            if key_id in domain.encryption_key_ids:
                return True
        return False

    def decrypt(self, cipher_text: bytes) -> bytes:
        assert isinstance(cipher_text, bytes)
        self.__verify_bootstrapped_and_registered()
        self.__update_config_by_elasped_time(MAX_ELASPED_TIME)

        cipher_text_blob, cfg = self.__crypto_context.deserialize(cipher_text)
        aad = self.__crypto_context.extract_unverified_aad(cipher_text).aad
        aad = self.__parse_cipher_text_AAD(aad)
        if not self.__is_key_id_decryption_viable(aad.cryptoKeyID):
            self.logger.warning("Ciphertext is no longer viable for decryption")
            raise NoValidUseDomainsForDecryptionError("Ciphertext is no longer viable for decryption")

        key = self.__get_key(aad.cryptoKeyID)
        pmKey = p.Key(p.SymmetricCipher(cfg.symm_cipher), key)
        plain_text, need_verification = self.__crypto_context.decrypt(pmKey, cipher_text_blob)
        if need_verification and not self.__verify_message(aad, cfg, cipher_text_blob, plain_text):
            self.logger.warning("Tampering of data detected, verification failed.")
            raise CoreCryptoError("Tampering of data detected, verification failed.")

        result = plain_text.data

        assert isinstance(result, bytes)
        return result




# coding: utf-8

"""
    Peacemakr

    This API describes the Peacemakr services, which enable seamless application layer encryption and verification.  # noqa: E501

    OpenAPI spec version: 1.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class KeyDerivationInstance(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'str',
        'service_ids': 'list[str]',
        'active': 'bool',
        'version': 'str',
        'base_url': 'str',
        'is_public': 'bool'
    }

    attribute_map = {
        'id': 'id',
        'service_ids': 'serviceIds',
        'active': 'active',
        'version': 'version',
        'base_url': 'baseUrl',
        'is_public': 'isPublic'
    }

    def __init__(self, id=None, service_ids=None, active=None, version=None, base_url=None, is_public=False):  # noqa: E501
        """KeyDerivationInstance - a model defined in Swagger"""  # noqa: E501

        self._id = None
        self._service_ids = None
        self._active = None
        self._version = None
        self._base_url = None
        self._is_public = None
        self.discriminator = None

        self.id = id
        self.service_ids = service_ids
        self.active = active
        self.version = version
        if base_url is not None:
            self.base_url = base_url
        if is_public is not None:
            self.is_public = is_public

    @property
    def id(self):
        """Gets the id of this KeyDerivationInstance.  # noqa: E501

        instance id (concrete instance)  # noqa: E501

        :return: The id of this KeyDerivationInstance.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this KeyDerivationInstance.

        instance id (concrete instance)  # noqa: E501

        :param id: The id of this KeyDerivationInstance.  # noqa: E501
        :type: str
        """
        if id is None:
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def service_ids(self):
        """Gets the service_ids of this KeyDerivationInstance.  # noqa: E501

        service id (virtual service id)  # noqa: E501

        :return: The service_ids of this KeyDerivationInstance.  # noqa: E501
        :rtype: list[str]
        """
        return self._service_ids

    @service_ids.setter
    def service_ids(self, service_ids):
        """Sets the service_ids of this KeyDerivationInstance.

        service id (virtual service id)  # noqa: E501

        :param service_ids: The service_ids of this KeyDerivationInstance.  # noqa: E501
        :type: list[str]
        """
        if service_ids is None:
            raise ValueError("Invalid value for `service_ids`, must not be `None`")  # noqa: E501

        self._service_ids = service_ids

    @property
    def active(self):
        """Gets the active of this KeyDerivationInstance.  # noqa: E501

        currently online and accepting requests for key derivation  # noqa: E501

        :return: The active of this KeyDerivationInstance.  # noqa: E501
        :rtype: bool
        """
        return self._active

    @active.setter
    def active(self, active):
        """Sets the active of this KeyDerivationInstance.

        currently online and accepting requests for key derivation  # noqa: E501

        :param active: The active of this KeyDerivationInstance.  # noqa: E501
        :type: bool
        """
        if active is None:
            raise ValueError("Invalid value for `active`, must not be `None`")  # noqa: E501

        self._active = active

    @property
    def version(self):
        """Gets the version of this KeyDerivationInstance.  # noqa: E501


        :return: The version of this KeyDerivationInstance.  # noqa: E501
        :rtype: str
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this KeyDerivationInstance.


        :param version: The version of this KeyDerivationInstance.  # noqa: E501
        :type: str
        """
        if version is None:
            raise ValueError("Invalid value for `version`, must not be `None`")  # noqa: E501

        self._version = version

    @property
    def base_url(self):
        """Gets the base_url of this KeyDerivationInstance.  # noqa: E501

        base URL from which this key deriver instance will respond to new key derivation job requests  # noqa: E501

        :return: The base_url of this KeyDerivationInstance.  # noqa: E501
        :rtype: str
        """
        return self._base_url

    @base_url.setter
    def base_url(self, base_url):
        """Sets the base_url of this KeyDerivationInstance.

        base URL from which this key deriver instance will respond to new key derivation job requests  # noqa: E501

        :param base_url: The base_url of this KeyDerivationInstance.  # noqa: E501
        :type: str
        """

        self._base_url = base_url

    @property
    def is_public(self):
        """Gets the is_public of this KeyDerivationInstance.  # noqa: E501

        if true then the key deriver is visible to every other organization  # noqa: E501

        :return: The is_public of this KeyDerivationInstance.  # noqa: E501
        :rtype: bool
        """
        return self._is_public

    @is_public.setter
    def is_public(self, is_public):
        """Sets the is_public of this KeyDerivationInstance.

        if true then the key deriver is visible to every other organization  # noqa: E501

        :param is_public: The is_public of this KeyDerivationInstance.  # noqa: E501
        :type: bool
        """

        self._is_public = is_public

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(KeyDerivationInstance, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, KeyDerivationInstance):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Peacemakr

    This API describes the Peacemakr services, which enable seamless application layer encryption and verification.  # noqa: E501

    OpenAPI spec version: 1.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from peacemakr.generated.models.api_key import APIKey  # noqa: F401,E501
from peacemakr.generated.models.contact import Contact  # noqa: F401,E501


class Organization(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'str',
        'name': 'str',
        'contacts': 'list[Contact]',
        'stripe_customer_id': 'str',
        'client_ids': 'list[str]',
        'api_keys': 'list[APIKey]',
        'crypto_config_id': 'str'
    }

    attribute_map = {
        'id': 'id',
        'name': 'name',
        'contacts': 'contacts',
        'stripe_customer_id': 'stripeCustomerId',
        'client_ids': 'clientIds',
        'api_keys': 'apiKeys',
        'crypto_config_id': 'cryptoConfigId'
    }

    def __init__(self, id=None, name=None, contacts=None, stripe_customer_id=None, client_ids=None, api_keys=None, crypto_config_id=None):  # noqa: E501
        """Organization - a model defined in Swagger"""  # noqa: E501

        self._id = None
        self._name = None
        self._contacts = None
        self._stripe_customer_id = None
        self._client_ids = None
        self._api_keys = None
        self._crypto_config_id = None
        self.discriminator = None

        self.id = id
        self.name = name
        self.contacts = contacts
        self.stripe_customer_id = stripe_customer_id
        self.client_ids = client_ids
        self.api_keys = api_keys
        self.crypto_config_id = crypto_config_id

    @property
    def id(self):
        """Gets the id of this Organization.  # noqa: E501


        :return: The id of this Organization.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Organization.


        :param id: The id of this Organization.  # noqa: E501
        :type: str
        """
        if id is None:
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def name(self):
        """Gets the name of this Organization.  # noqa: E501


        :return: The name of this Organization.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Organization.


        :param name: The name of this Organization.  # noqa: E501
        :type: str
        """
        if name is None:
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def contacts(self):
        """Gets the contacts of this Organization.  # noqa: E501


        :return: The contacts of this Organization.  # noqa: E501
        :rtype: list[Contact]
        """
        return self._contacts

    @contacts.setter
    def contacts(self, contacts):
        """Sets the contacts of this Organization.


        :param contacts: The contacts of this Organization.  # noqa: E501
        :type: list[Contact]
        """
        if contacts is None:
            raise ValueError("Invalid value for `contacts`, must not be `None`")  # noqa: E501

        self._contacts = contacts

    @property
    def stripe_customer_id(self):
        """Gets the stripe_customer_id of this Organization.  # noqa: E501

        Identifies the the customer in Stripe associated with this org  # noqa: E501

        :return: The stripe_customer_id of this Organization.  # noqa: E501
        :rtype: str
        """
        return self._stripe_customer_id

    @stripe_customer_id.setter
    def stripe_customer_id(self, stripe_customer_id):
        """Sets the stripe_customer_id of this Organization.

        Identifies the the customer in Stripe associated with this org  # noqa: E501

        :param stripe_customer_id: The stripe_customer_id of this Organization.  # noqa: E501
        :type: str
        """
        if stripe_customer_id is None:
            raise ValueError("Invalid value for `stripe_customer_id`, must not be `None`")  # noqa: E501

        self._stripe_customer_id = stripe_customer_id

    @property
    def client_ids(self):
        """Gets the client_ids of this Organization.  # noqa: E501

        Array of client id's registered to this org  # noqa: E501

        :return: The client_ids of this Organization.  # noqa: E501
        :rtype: list[str]
        """
        return self._client_ids

    @client_ids.setter
    def client_ids(self, client_ids):
        """Sets the client_ids of this Organization.

        Array of client id's registered to this org  # noqa: E501

        :param client_ids: The client_ids of this Organization.  # noqa: E501
        :type: list[str]
        """
        if client_ids is None:
            raise ValueError("Invalid value for `client_ids`, must not be `None`")  # noqa: E501

        self._client_ids = client_ids

    @property
    def api_keys(self):
        """Gets the api_keys of this Organization.  # noqa: E501

        Array of api keys registered to this org  # noqa: E501

        :return: The api_keys of this Organization.  # noqa: E501
        :rtype: list[APIKey]
        """
        return self._api_keys

    @api_keys.setter
    def api_keys(self, api_keys):
        """Sets the api_keys of this Organization.

        Array of api keys registered to this org  # noqa: E501

        :param api_keys: The api_keys of this Organization.  # noqa: E501
        :type: list[APIKey]
        """
        if api_keys is None:
            raise ValueError("Invalid value for `api_keys`, must not be `None`")  # noqa: E501

        self._api_keys = api_keys

    @property
    def crypto_config_id(self):
        """Gets the crypto_config_id of this Organization.  # noqa: E501

        cryptoconfigId of this org  # noqa: E501

        :return: The crypto_config_id of this Organization.  # noqa: E501
        :rtype: str
        """
        return self._crypto_config_id

    @crypto_config_id.setter
    def crypto_config_id(self, crypto_config_id):
        """Sets the crypto_config_id of this Organization.

        cryptoconfigId of this org  # noqa: E501

        :param crypto_config_id: The crypto_config_id of this Organization.  # noqa: E501
        :type: str
        """
        if crypto_config_id is None:
            raise ValueError("Invalid value for `crypto_config_id`, must not be `None`")  # noqa: E501

        self._crypto_config_id = crypto_config_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Organization, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Organization):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Peacemakr

    This API describes the Peacemakr services, which enable seamless application layer encryption and verification.  # noqa: E501

    OpenAPI spec version: 1.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import unittest

import peacemakr
from peacemakr.generated.api.key_derivation_service_registry_api import KeyDerivationServiceRegistryApi  # noqa: E501
from peacemakr.generated.rest import ApiException


class TestKeyDerivationServiceRegistryApi(unittest.TestCase):
    """KeyDerivationServiceRegistryApi unit test stubs"""

    def setUp(self):
        self.api = peacemakr.generated.api.key_derivation_service_registry_api.KeyDerivationServiceRegistryApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_add_key_derivation_service_instance(self):
        """Test case for add_key_derivation_service_instance

        Register a new KeyDerivationServiceInstance  # noqa: E501
        """
        pass

    def test_delete_key_derivation_service_instance(self):
        """Test case for delete_key_derivation_service_instance

        Activate or deactivate an existing KeyDerivationServiceInstance  # noqa: E501
        """
        pass

    def test_get_all_org_key_derivation_service_instances(self):
        """Test case for get_all_org_key_derivation_service_instances

        Get the all key derivers registerd to org  # noqa: E501
        """
        pass

    def test_get_all_shared_key_derivation_service_instances(self):
        """Test case for get_all_shared_key_derivation_service_instances

        Get the all key derivers that the org has access to - including shared cloud instances  # noqa: E501
        """
        pass

    def test_get_key_derivation_service_instance(self):
        """Test case for get_key_derivation_service_instance

        Get the keyderiver details by id  # noqa: E501
        """
        pass

    def test_heartbeat_key_derivation_service_instance(self):
        """Test case for heartbeat_key_derivation_service_instance

        Heatbeat from the given key derivation service instance  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

# flake8: noqa

"""
    Peacemakr

    This API describes the Peacemakr services, which enable seamless application layer encryption and verification.  # noqa: E501

    OpenAPI spec version: 1.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

# import apis into sdk package
from peacemakr.generated.api.client_api import ClientApi
from peacemakr.generated.api.crypto_config_api import CryptoConfigApi
from peacemakr.generated.api.key_derivation_service_registry_api import KeyDerivationServiceRegistryApi
from peacemakr.generated.api.key_service_api import KeyServiceApi
from peacemakr.generated.api.login_api import LoginApi
from peacemakr.generated.api.org_api import OrgApi
from peacemakr.generated.api.phone_home_api import PhoneHomeApi
from peacemakr.generated.api.server_management_api import ServerManagementApi

# import ApiClient
from peacemakr.generated.api_client import ApiClient
from peacemakr.generated.configuration import Configuration
# import models into sdk package
from peacemakr.generated.models.api_key import APIKey
from peacemakr.generated.models.client import Client
from peacemakr.generated.models.contact import Contact
from peacemakr.generated.models.crypto_config import CryptoConfig
from peacemakr.generated.models.encrypted_symmetric_key import EncryptedSymmetricKey
from peacemakr.generated.models.error_response import ErrorResponse
from peacemakr.generated.models.heatbeat_response import HeatbeatResponse
from peacemakr.generated.models.key_derivation_instance import KeyDerivationInstance
from peacemakr.generated.models.log import Log
from peacemakr.generated.models.login_response import LoginResponse
from peacemakr.generated.models.organization import Organization
from peacemakr.generated.models.public_key import PublicKey
from peacemakr.generated.models.symmetric_key_request import SymmetricKeyRequest
from peacemakr.generated.models.symmetric_key_use_domain import SymmetricKeyUseDomain



# coding: utf-8

"""
    Peacemakr

    This API describes the Peacemakr services, which enable seamless application layer encryption and verification.  # noqa: E501

    OpenAPI spec version: 1.0.0

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from setuptools import setup, find_packages  # noqa: H301
import setuptools.command.build_py
import distutils.cmd
import distutils.log
import subprocess
import setuptools
import sys
import os
import shutil
import platform

NAME = "peacemakr"
VERSION = "0.0.2"

# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

REQUIRES = [
    "certifi>=2017.4.17",
    "python-dateutil>=2.1",
    "six>=1.10",
    "urllib3>=1.23",
    "distro",
]

CORE_CRYPTO_URL_BASE="https://github.com/peacemakr-io/peacemakr-core-crypto/releases/download/"
CORE_CRYPTO_VERSION="v0.2.0"

class InstallCoreCryptoCommand(distutils.cmd.Command):
  """A custom command to run Pylint on all Python source files."""

  description = 'run Pylint on Python source files'
  user_options = []
  def initialize_options(self):
    """Set default values for options."""
    pass

  def finalize_options(self):
    """Post-process options."""
    pass

  def _execute_command(self, command, err_msg=""):
    try:
        subprocess.check_output(command, stderr=subprocess.STDOUT)
    except Exception as e:
        sys.exit("Running command %s failed. %s. %s" % (" ".join(command), e, err_msg))

  def _execute_shell_command(self, command, err_msg=""):
      ret_code = os.system(" ".join(command))

      if ret_code != 0:
          sys.exit("Running shell command %s failed with error code %s. %s" % (" ".join(command), ret_code, err_msg))

  def _is_pyversion_in_range(self, tuple, major_version, minor_version):
    return tuple[0] == major_version and tuple[1] == minor_version

  def _install_from_artifact(self, tar_filename, core_crypto_so_filename, core_crypto_shared_filename, core_crypto_cpp_shared_filename):
    site_package_dir = next(p for p in sys.path if 'site-packages' in p)
    get_command = "wget -q " + CORE_CRYPTO_URL_BASE + CORE_CRYPTO_VERSION + "/" + tar_filename
    unzip_command = "tar -zxvf " + tar_filename

    self._execute_command(get_command.split(" "))
    self._execute_command(unzip_command.split(" "))

    # copying file
    shutil.copyfile(core_crypto_so_filename, site_package_dir+"/"+core_crypto_so_filename)
    shutil.copyfile("lib/"+core_crypto_shared_filename, "/usr/local/lib/"+core_crypto_shared_filename)
    shutil.copyfile("lib/"+core_crypto_cpp_shared_filename, "/usr/local/lib/"+core_crypto_cpp_shared_filename)

  def _virtualenv_enabled(self):
    """Checks virtualenv is enabled or not"""
    return hasattr(sys, 'real_prefix')

  def _install_from_source(self):
    site_package_dir = "none"
    if self._virtualenv_enabled():
        site_package_dir = next(p for p in sys.path if 'site-packages' in p)
    # commands to run
    git_clone_command = ["git" ,"clone", "https://github.com/peacemakr-io/peacemakr-core-crypto.git"]
    git_checkout_tag = ["cd", "peacemakr-core-crypto", "&&", "git", "checkout", "tags/"+CORE_CRYPTO_VERSION]
    rm_clone_command = ["rm", "-rf", "peacemakr-core-crypto"]
    install_command = ["cd", "peacemakr-core-crypto/bin", "&&", "./release-python.sh", "local", site_package_dir, "release"]
    check_openssl_installed = ["openssl", "version"]
    check_cmake_installed = ["cmake", "--version"]

    # clone the repo
    self.announce(
    'Cloning core-crypto @: %s' % ("https://github.com/peacemakr-io/peacemakr-core-crypto.git"),
    level=distutils.log.INFO)
    self._execute_command(git_clone_command)

    # checkout lastest stable core-crypto tag
    self.announce(
        'Checking out tag '+CORE_CRYPTO_VERSION,
        level=distutils.log.INFO)
    self._execute_shell_command(git_checkout_tag)

    # check openssl and cmake
    self.announce("Checking CMake and OpenSSL is installed", level=distutils.log.INFO)
    self._execute_shell_command(check_cmake_installed, err_msg="CMake not installed. CMake 3.15+ required for this library.\n\n\n \
        Please refer to Core-crypto for installation: https://github.com/peacemakr-io/peacemakr-core-crypto/")
    self._execute_shell_command(check_openssl_installed, err_msg="OpenSSL not installed. Openssl 1.1+ required for this library.\n\n\n \
        Please refer to Core-crypto for installation: https://github.com/peacemakr-io/peacemakr-core-crypto/")

    # running release commands
    self.announce(
    'Installing core-crytpo into %s %s' % (str(site_package_dir), " ".join(install_command)),
    level=distutils.log.INFO)
    self._execute_shell_command(install_command, err_msg="Installation from source failed. Please refer to Core-crypto for installation: https://github.com/peacemakr-io/peacemakr-core-crypto/")

    # remove peacemakr-core-crypto
    self.announce(
    'Removing core-crytpo folder',
    level=distutils.log.INFO)
    self._execute_command(rm_clone_command)

  def run(self):
    """Build core-crypto
    Two methods to build core-crypto python library
    1. From Artifact: If user is using a OS/Arch/Python version that fits the deployed artifact,
       the command simply fetch and deploy the pre-built library
    2. From Source: If the version doesn't match, but is still macos/linux. The command
       clone the core-crypto from specific tag and built from source.

    # TODO: Remind user to set LD_LIBRARY_PATH for Ubuntu in README
    """
    os_type = sys.platform
    machine_type = platform.machine()
    python_version = platform.python_version_tuple()
    tar_filename = ""
    core_crypto_so_filename = ""
    core_crypto_shared_filename = ""
    core_crypto_cpp_shared_filename = ""

    if os_type == "darwin":
        # user is using macos
        if machine_type == "x86_64" and self._is_pyversion_in_range(python_version, '3', '7'):
            # install from artifact
            tar_filename = "peacemakr-core-crypto-python-macos-x86_64.tar.gz"
            core_crypto_so_filename = "peacemakr_core_crypto_python.cpython-37m-darwin.so"
            core_crypto_shared_filename = "libpeacemakr-core-crypto.dylib"
            core_crypto_cpp_shared_filename = "libpeacemakr-core-crypto-cpp.dylib"

            self._install_from_artifact(tar_filename, core_crypto_so_filename, core_crypto_shared_filename, core_crypto_cpp_shared_filename)

        else:
            # install from src
            self.announce("Installing core-crypto from source", level=distutils.log.INFO)
            self._install_from_source()

    elif os_type == "linux":
        # user is using linux
        import distro
        linux_distribution = distro.linux_distribution(full_distribution_name=False)[0]
        
        if linux_distribution in ["debian", "ubuntu"] and machine_type == "x86_64" and self._is_pyversion_in_range(python_version, '3', '6'):
            # install from artifact
            tar_filename = "peacemakr-core-crypto-python-ubuntu-x86_64.tar.gz"
            core_crypto_so_filename = "peacemakr_core_crypto_python.cpython-36m-x86_64-linux-gnu.so"
            core_crypto_shared_filename = "libpeacemakr-core-crypto.so"
            core_crypto_cpp_shared_filename = "libpeacemakr-core-crypto-cpp.so"

            self._install_from_artifact(tar_filename, core_crypto_so_filename, core_crypto_shared_filename, core_crypto_cpp_shared_filename)

        else:
            # install from src
            self.announce("Installing in Non-Debian Linux, installing core-crypto from source", level=distutils.log.INFO)
            self._install_from_source()
    else:
        sys.exit("Error: OS not supported. We only support Ubuntu and MacOS at the moment.")

class BuildPyCommand(setuptools.command.build_py.build_py):
  """Custom build command."""

  def run(self):
    self.run_command('install_core_crypto')
    setuptools.command.build_py.build_py.run(self)

setup(
    cmdclass={
        'install_core_crypto': InstallCoreCryptoCommand,
        'build_py': BuildPyCommand,
    },
    name=NAME,
    version=VERSION,
    description="Peacemakr",
    python_requires='>=3.6',
    author_email="",
    url="",
    keywords=["Peacemakr"],
    install_requires=REQUIRES,
    packages=find_packages(),
    include_package_data=True,
    long_description="""\
    This API describes the Peacemakr services, which enable seamless application layer encryption and verification.  # noqa: E501
    """,
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Security",
        "Topic :: Software Development :: Libraries",
    ]
)



# coding: utf-8

"""
    Peacemakr

    This API describes the Peacemakr services, which enable seamless application layer encryption and verification.  # noqa: E501

    OpenAPI spec version: 1.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import unittest

import peacemakr
from peacemakr.generated.api.server_management_api import ServerManagementApi  # noqa: E501
from peacemakr.generated.rest import ApiException


class TestServerManagementApi(unittest.TestCase):
    """ServerManagementApi unit test stubs"""

    def setUp(self):
        self.api = peacemakr.generated.api.server_management_api.ServerManagementApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_health_get(self):
        """Test case for health_get

        See if the server is healthy  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Peacemakr

    This API describes the Peacemakr services, which enable seamless application layer encryption and verification.  # noqa: E501

    OpenAPI spec version: 1.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from peacemakr.generated.models.contact import Contact  # noqa: F401,E501


class APIKey(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'key': 'str',
        'org_id': 'str',
        'creation_time': 'int',
        'authorized_use_domains': 'list[str]',
        'creator': 'Contact'
    }

    attribute_map = {
        'key': 'key',
        'org_id': 'orgId',
        'creation_time': 'creationTime',
        'authorized_use_domains': 'authorizedUseDomains',
        'creator': 'creator'
    }

    def __init__(self, key=None, org_id=None, creation_time=None, authorized_use_domains=None, creator=None):  # noqa: E501
        """APIKey - a model defined in Swagger"""  # noqa: E501

        self._key = None
        self._org_id = None
        self._creation_time = None
        self._authorized_use_domains = None
        self._creator = None
        self.discriminator = None

        self.key = key
        self.org_id = org_id
        if creation_time is not None:
            self.creation_time = creation_time
        if authorized_use_domains is not None:
            self.authorized_use_domains = authorized_use_domains
        if creator is not None:
            self.creator = creator

    @property
    def key(self):
        """Gets the key of this APIKey.  # noqa: E501


        :return: The key of this APIKey.  # noqa: E501
        :rtype: str
        """
        return self._key

    @key.setter
    def key(self, key):
        """Sets the key of this APIKey.


        :param key: The key of this APIKey.  # noqa: E501
        :type: str
        """
        if key is None:
            raise ValueError("Invalid value for `key`, must not be `None`")  # noqa: E501

        self._key = key

    @property
    def org_id(self):
        """Gets the org_id of this APIKey.  # noqa: E501


        :return: The org_id of this APIKey.  # noqa: E501
        :rtype: str
        """
        return self._org_id

    @org_id.setter
    def org_id(self, org_id):
        """Sets the org_id of this APIKey.


        :param org_id: The org_id of this APIKey.  # noqa: E501
        :type: str
        """
        if org_id is None:
            raise ValueError("Invalid value for `org_id`, must not be `None`")  # noqa: E501

        self._org_id = org_id

    @property
    def creation_time(self):
        """Gets the creation_time of this APIKey.  # noqa: E501


        :return: The creation_time of this APIKey.  # noqa: E501
        :rtype: int
        """
        return self._creation_time

    @creation_time.setter
    def creation_time(self, creation_time):
        """Sets the creation_time of this APIKey.


        :param creation_time: The creation_time of this APIKey.  # noqa: E501
        :type: int
        """

        self._creation_time = creation_time

    @property
    def authorized_use_domains(self):
        """Gets the authorized_use_domains of this APIKey.  # noqa: E501

        list of useDomains that APIKey has access to  # noqa: E501

        :return: The authorized_use_domains of this APIKey.  # noqa: E501
        :rtype: list[str]
        """
        return self._authorized_use_domains

    @authorized_use_domains.setter
    def authorized_use_domains(self, authorized_use_domains):
        """Sets the authorized_use_domains of this APIKey.

        list of useDomains that APIKey has access to  # noqa: E501

        :param authorized_use_domains: The authorized_use_domains of this APIKey.  # noqa: E501
        :type: list[str]
        """

        self._authorized_use_domains = authorized_use_domains

    @property
    def creator(self):
        """Gets the creator of this APIKey.  # noqa: E501


        :return: The creator of this APIKey.  # noqa: E501
        :rtype: Contact
        """
        return self._creator

    @creator.setter
    def creator(self, creator):
        """Sets the creator of this APIKey.


        :param creator: The creator of this APIKey.  # noqa: E501
        :type: Contact
        """

        self._creator = creator

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(APIKey, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, APIKey):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Peacemakr

    This API describes the Peacemakr services, which enable seamless application layer encryption and verification.  # noqa: E501

    OpenAPI spec version: 1.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class SymmetricKeyUseDomain(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'str',
        'owner_org_id': 'str',
        'name': 'str',
        'creation_time': 'int',
        'symmetric_key_inception_ttl': 'int',
        'symmetric_key_encryption_use_ttl': 'int',
        'symmetric_key_encryption_allowed': 'bool',
        'symmetric_key_decryption_use_ttl': 'int',
        'symmetric_key_decryption_allowed': 'bool',
        'symmetric_key_retention_use_ttl': 'int',
        'symmetric_key_length': 'int',
        'symmetric_key_encryption_alg': 'str',
        'encrypting_packaged_ciphertext_version': 'int',
        'symmetric_key_derivation_service_id': 'str',
        'encryption_key_ids': 'list[str]',
        'endable_kds_fallback_to_cloud': 'bool',
        'require_signed_key_delivery': 'bool',
        'digest_algorithm': 'str'
    }

    attribute_map = {
        'id': 'id',
        'owner_org_id': 'ownerOrgId',
        'name': 'name',
        'creation_time': 'creationTime',
        'symmetric_key_inception_ttl': 'symmetricKeyInceptionTTL',
        'symmetric_key_encryption_use_ttl': 'symmetricKeyEncryptionUseTTL',
        'symmetric_key_encryption_allowed': 'symmetricKeyEncryptionAllowed',
        'symmetric_key_decryption_use_ttl': 'symmetricKeyDecryptionUseTTL',
        'symmetric_key_decryption_allowed': 'symmetricKeyDecryptionAllowed',
        'symmetric_key_retention_use_ttl': 'symmetricKeyRetentionUseTTL',
        'symmetric_key_length': 'symmetricKeyLength',
        'symmetric_key_encryption_alg': 'symmetricKeyEncryptionAlg',
        'encrypting_packaged_ciphertext_version': 'encryptingPackagedCiphertextVersion',
        'symmetric_key_derivation_service_id': 'symmetricKeyDerivationServiceId',
        'encryption_key_ids': 'encryptionKeyIds',
        'endable_kds_fallback_to_cloud': 'endableKDSFallbackToCloud',
        'require_signed_key_delivery': 'requireSignedKeyDelivery',
        'digest_algorithm': 'digestAlgorithm'
    }

    def __init__(self, id=None, owner_org_id=None, name=None, creation_time=None, symmetric_key_inception_ttl=None, symmetric_key_encryption_use_ttl=None, symmetric_key_encryption_allowed=None, symmetric_key_decryption_use_ttl=None, symmetric_key_decryption_allowed=None, symmetric_key_retention_use_ttl=None, symmetric_key_length=None, symmetric_key_encryption_alg='CHACHA20_POLY1305', encrypting_packaged_ciphertext_version=None, symmetric_key_derivation_service_id=None, encryption_key_ids=None, endable_kds_fallback_to_cloud=None, require_signed_key_delivery=None, digest_algorithm='SHA_256'):  # noqa: E501
        """SymmetricKeyUseDomain - a model defined in Swagger"""  # noqa: E501

        self._id = None
        self._owner_org_id = None
        self._name = None
        self._creation_time = None
        self._symmetric_key_inception_ttl = None
        self._symmetric_key_encryption_use_ttl = None
        self._symmetric_key_encryption_allowed = None
        self._symmetric_key_decryption_use_ttl = None
        self._symmetric_key_decryption_allowed = None
        self._symmetric_key_retention_use_ttl = None
        self._symmetric_key_length = None
        self._symmetric_key_encryption_alg = None
        self._encrypting_packaged_ciphertext_version = None
        self._symmetric_key_derivation_service_id = None
        self._encryption_key_ids = None
        self._endable_kds_fallback_to_cloud = None
        self._require_signed_key_delivery = None
        self._digest_algorithm = None
        self.discriminator = None

        self.id = id
        self.owner_org_id = owner_org_id
        if name is not None:
            self.name = name
        self.creation_time = creation_time
        self.symmetric_key_inception_ttl = symmetric_key_inception_ttl
        self.symmetric_key_encryption_use_ttl = symmetric_key_encryption_use_ttl
        if symmetric_key_encryption_allowed is not None:
            self.symmetric_key_encryption_allowed = symmetric_key_encryption_allowed
        self.symmetric_key_decryption_use_ttl = symmetric_key_decryption_use_ttl
        if symmetric_key_decryption_allowed is not None:
            self.symmetric_key_decryption_allowed = symmetric_key_decryption_allowed
        self.symmetric_key_retention_use_ttl = symmetric_key_retention_use_ttl
        self.symmetric_key_length = symmetric_key_length
        self.symmetric_key_encryption_alg = symmetric_key_encryption_alg
        self.encrypting_packaged_ciphertext_version = encrypting_packaged_ciphertext_version
        self.symmetric_key_derivation_service_id = symmetric_key_derivation_service_id
        self.encryption_key_ids = encryption_key_ids
        self.endable_kds_fallback_to_cloud = endable_kds_fallback_to_cloud
        self.require_signed_key_delivery = require_signed_key_delivery
        if digest_algorithm is not None:
            self.digest_algorithm = digest_algorithm

    @property
    def id(self):
        """Gets the id of this SymmetricKeyUseDomain.  # noqa: E501


        :return: The id of this SymmetricKeyUseDomain.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this SymmetricKeyUseDomain.


        :param id: The id of this SymmetricKeyUseDomain.  # noqa: E501
        :type: str
        """
        if id is None:
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def owner_org_id(self):
        """Gets the owner_org_id of this SymmetricKeyUseDomain.  # noqa: E501

        the org id of the organization that owns these symmetric keys  # noqa: E501

        :return: The owner_org_id of this SymmetricKeyUseDomain.  # noqa: E501
        :rtype: str
        """
        return self._owner_org_id

    @owner_org_id.setter
    def owner_org_id(self, owner_org_id):
        """Sets the owner_org_id of this SymmetricKeyUseDomain.

        the org id of the organization that owns these symmetric keys  # noqa: E501

        :param owner_org_id: The owner_org_id of this SymmetricKeyUseDomain.  # noqa: E501
        :type: str
        """
        if owner_org_id is None:
            raise ValueError("Invalid value for `owner_org_id`, must not be `None`")  # noqa: E501

        self._owner_org_id = owner_org_id

    @property
    def name(self):
        """Gets the name of this SymmetricKeyUseDomain.  # noqa: E501


        :return: The name of this SymmetricKeyUseDomain.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this SymmetricKeyUseDomain.


        :param name: The name of this SymmetricKeyUseDomain.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def creation_time(self):
        """Gets the creation_time of this SymmetricKeyUseDomain.  # noqa: E501


        :return: The creation_time of this SymmetricKeyUseDomain.  # noqa: E501
        :rtype: int
        """
        return self._creation_time

    @creation_time.setter
    def creation_time(self, creation_time):
        """Sets the creation_time of this SymmetricKeyUseDomain.


        :param creation_time: The creation_time of this SymmetricKeyUseDomain.  # noqa: E501
        :type: int
        """
        if creation_time is None:
            raise ValueError("Invalid value for `creation_time`, must not be `None`")  # noqa: E501

        self._creation_time = creation_time

    @property
    def symmetric_key_inception_ttl(self):
        """Gets the symmetric_key_inception_ttl of this SymmetricKeyUseDomain.  # noqa: E501

        number of seconds since key creation that the key will be available for encryption  # noqa: E501

        :return: The symmetric_key_inception_ttl of this SymmetricKeyUseDomain.  # noqa: E501
        :rtype: int
        """
        return self._symmetric_key_inception_ttl

    @symmetric_key_inception_ttl.setter
    def symmetric_key_inception_ttl(self, symmetric_key_inception_ttl):
        """Sets the symmetric_key_inception_ttl of this SymmetricKeyUseDomain.

        number of seconds since key creation that the key will be available for encryption  # noqa: E501

        :param symmetric_key_inception_ttl: The symmetric_key_inception_ttl of this SymmetricKeyUseDomain.  # noqa: E501
        :type: int
        """
        if symmetric_key_inception_ttl is None:
            raise ValueError("Invalid value for `symmetric_key_inception_ttl`, must not be `None`")  # noqa: E501

        self._symmetric_key_inception_ttl = symmetric_key_inception_ttl

    @property
    def symmetric_key_encryption_use_ttl(self):
        """Gets the symmetric_key_encryption_use_ttl of this SymmetricKeyUseDomain.  # noqa: E501

        number of seconds since key creation that the key will be available for encryption  # noqa: E501

        :return: The symmetric_key_encryption_use_ttl of this SymmetricKeyUseDomain.  # noqa: E501
        :rtype: int
        """
        return self._symmetric_key_encryption_use_ttl

    @symmetric_key_encryption_use_ttl.setter
    def symmetric_key_encryption_use_ttl(self, symmetric_key_encryption_use_ttl):
        """Sets the symmetric_key_encryption_use_ttl of this SymmetricKeyUseDomain.

        number of seconds since key creation that the key will be available for encryption  # noqa: E501

        :param symmetric_key_encryption_use_ttl: The symmetric_key_encryption_use_ttl of this SymmetricKeyUseDomain.  # noqa: E501
        :type: int
        """
        if symmetric_key_encryption_use_ttl is None:
            raise ValueError("Invalid value for `symmetric_key_encryption_use_ttl`, must not be `None`")  # noqa: E501

        self._symmetric_key_encryption_use_ttl = symmetric_key_encryption_use_ttl

    @property
    def symmetric_key_encryption_allowed(self):
        """Gets the symmetric_key_encryption_allowed of this SymmetricKeyUseDomain.  # noqa: E501

        whether this use domain is available for encryption; if false, these keys should not be used for encrypting new messages  # noqa: E501

        :return: The symmetric_key_encryption_allowed of this SymmetricKeyUseDomain.  # noqa: E501
        :rtype: bool
        """
        return self._symmetric_key_encryption_allowed

    @symmetric_key_encryption_allowed.setter
    def symmetric_key_encryption_allowed(self, symmetric_key_encryption_allowed):
        """Sets the symmetric_key_encryption_allowed of this SymmetricKeyUseDomain.

        whether this use domain is available for encryption; if false, these keys should not be used for encrypting new messages  # noqa: E501

        :param symmetric_key_encryption_allowed: The symmetric_key_encryption_allowed of this SymmetricKeyUseDomain.  # noqa: E501
        :type: bool
        """

        self._symmetric_key_encryption_allowed = symmetric_key_encryption_allowed

    @property
    def symmetric_key_decryption_use_ttl(self):
        """Gets the symmetric_key_decryption_use_ttl of this SymmetricKeyUseDomain.  # noqa: E501

        number of seconds since key creation that the key will be available for decryption  # noqa: E501

        :return: The symmetric_key_decryption_use_ttl of this SymmetricKeyUseDomain.  # noqa: E501
        :rtype: int
        """
        return self._symmetric_key_decryption_use_ttl

    @symmetric_key_decryption_use_ttl.setter
    def symmetric_key_decryption_use_ttl(self, symmetric_key_decryption_use_ttl):
        """Sets the symmetric_key_decryption_use_ttl of this SymmetricKeyUseDomain.

        number of seconds since key creation that the key will be available for decryption  # noqa: E501

        :param symmetric_key_decryption_use_ttl: The symmetric_key_decryption_use_ttl of this SymmetricKeyUseDomain.  # noqa: E501
        :type: int
        """
        if symmetric_key_decryption_use_ttl is None:
            raise ValueError("Invalid value for `symmetric_key_decryption_use_ttl`, must not be `None`")  # noqa: E501

        self._symmetric_key_decryption_use_ttl = symmetric_key_decryption_use_ttl

    @property
    def symmetric_key_decryption_allowed(self):
        """Gets the symmetric_key_decryption_allowed of this SymmetricKeyUseDomain.  # noqa: E501

        whether this use domain is available for decryption; if false, these keys should not be used for decrypting messages  # noqa: E501

        :return: The symmetric_key_decryption_allowed of this SymmetricKeyUseDomain.  # noqa: E501
        :rtype: bool
        """
        return self._symmetric_key_decryption_allowed

    @symmetric_key_decryption_allowed.setter
    def symmetric_key_decryption_allowed(self, symmetric_key_decryption_allowed):
        """Sets the symmetric_key_decryption_allowed of this SymmetricKeyUseDomain.

        whether this use domain is available for decryption; if false, these keys should not be used for decrypting messages  # noqa: E501

        :param symmetric_key_decryption_allowed: The symmetric_key_decryption_allowed of this SymmetricKeyUseDomain.  # noqa: E501
        :type: bool
        """

        self._symmetric_key_decryption_allowed = symmetric_key_decryption_allowed

    @property
    def symmetric_key_retention_use_ttl(self):
        """Gets the symmetric_key_retention_use_ttl of this SymmetricKeyUseDomain.  # noqa: E501

        number of seconds since key creation that the key will be available for retention purposes  # noqa: E501

        :return: The symmetric_key_retention_use_ttl of this SymmetricKeyUseDomain.  # noqa: E501
        :rtype: int
        """
        return self._symmetric_key_retention_use_ttl

    @symmetric_key_retention_use_ttl.setter
    def symmetric_key_retention_use_ttl(self, symmetric_key_retention_use_ttl):
        """Sets the symmetric_key_retention_use_ttl of this SymmetricKeyUseDomain.

        number of seconds since key creation that the key will be available for retention purposes  # noqa: E501

        :param symmetric_key_retention_use_ttl: The symmetric_key_retention_use_ttl of this SymmetricKeyUseDomain.  # noqa: E501
        :type: int
        """
        if symmetric_key_retention_use_ttl is None:
            raise ValueError("Invalid value for `symmetric_key_retention_use_ttl`, must not be `None`")  # noqa: E501

        self._symmetric_key_retention_use_ttl = symmetric_key_retention_use_ttl

    @property
    def symmetric_key_length(self):
        """Gets the symmetric_key_length of this SymmetricKeyUseDomain.  # noqa: E501

        the number of bits of all symmetric keys in this use domain  # noqa: E501

        :return: The symmetric_key_length of this SymmetricKeyUseDomain.  # noqa: E501
        :rtype: int
        """
        return self._symmetric_key_length

    @symmetric_key_length.setter
    def symmetric_key_length(self, symmetric_key_length):
        """Sets the symmetric_key_length of this SymmetricKeyUseDomain.

        the number of bits of all symmetric keys in this use domain  # noqa: E501

        :param symmetric_key_length: The symmetric_key_length of this SymmetricKeyUseDomain.  # noqa: E501
        :type: int
        """
        if symmetric_key_length is None:
            raise ValueError("Invalid value for `symmetric_key_length`, must not be `None`")  # noqa: E501

        self._symmetric_key_length = symmetric_key_length

    @property
    def symmetric_key_encryption_alg(self):
        """Gets the symmetric_key_encryption_alg of this SymmetricKeyUseDomain.  # noqa: E501

        the specific encryption alg to encrypt new plaintexts for application layer encryption operations  # noqa: E501

        :return: The symmetric_key_encryption_alg of this SymmetricKeyUseDomain.  # noqa: E501
        :rtype: str
        """
        return self._symmetric_key_encryption_alg

    @symmetric_key_encryption_alg.setter
    def symmetric_key_encryption_alg(self, symmetric_key_encryption_alg):
        """Sets the symmetric_key_encryption_alg of this SymmetricKeyUseDomain.

        the specific encryption alg to encrypt new plaintexts for application layer encryption operations  # noqa: E501

        :param symmetric_key_encryption_alg: The symmetric_key_encryption_alg of this SymmetricKeyUseDomain.  # noqa: E501
        :type: str
        """
        if symmetric_key_encryption_alg is None:
            raise ValueError("Invalid value for `symmetric_key_encryption_alg`, must not be `None`")  # noqa: E501

        self._symmetric_key_encryption_alg = symmetric_key_encryption_alg

    @property
    def encrypting_packaged_ciphertext_version(self):
        """Gets the encrypting_packaged_ciphertext_version of this SymmetricKeyUseDomain.  # noqa: E501

        after encrypting new plaintexts, package the ciphertext with this version of the packaged ciphertext  # noqa: E501

        :return: The encrypting_packaged_ciphertext_version of this SymmetricKeyUseDomain.  # noqa: E501
        :rtype: int
        """
        return self._encrypting_packaged_ciphertext_version

    @encrypting_packaged_ciphertext_version.setter
    def encrypting_packaged_ciphertext_version(self, encrypting_packaged_ciphertext_version):
        """Sets the encrypting_packaged_ciphertext_version of this SymmetricKeyUseDomain.

        after encrypting new plaintexts, package the ciphertext with this version of the packaged ciphertext  # noqa: E501

        :param encrypting_packaged_ciphertext_version: The encrypting_packaged_ciphertext_version of this SymmetricKeyUseDomain.  # noqa: E501
        :type: int
        """
        if encrypting_packaged_ciphertext_version is None:
            raise ValueError("Invalid value for `encrypting_packaged_ciphertext_version`, must not be `None`")  # noqa: E501

        self._encrypting_packaged_ciphertext_version = encrypting_packaged_ciphertext_version

    @property
    def symmetric_key_derivation_service_id(self):
        """Gets the symmetric_key_derivation_service_id of this SymmetricKeyUseDomain.  # noqa: E501

        the symmetric key derivation serivce id that can derive and wrap these keys  # noqa: E501

        :return: The symmetric_key_derivation_service_id of this SymmetricKeyUseDomain.  # noqa: E501
        :rtype: str
        """
        return self._symmetric_key_derivation_service_id

    @symmetric_key_derivation_service_id.setter
    def symmetric_key_derivation_service_id(self, symmetric_key_derivation_service_id):
        """Sets the symmetric_key_derivation_service_id of this SymmetricKeyUseDomain.

        the symmetric key derivation serivce id that can derive and wrap these keys  # noqa: E501

        :param symmetric_key_derivation_service_id: The symmetric_key_derivation_service_id of this SymmetricKeyUseDomain.  # noqa: E501
        :type: str
        """
        if symmetric_key_derivation_service_id is None:
            raise ValueError("Invalid value for `symmetric_key_derivation_service_id`, must not be `None`")  # noqa: E501

        self._symmetric_key_derivation_service_id = symmetric_key_derivation_service_id

    @property
    def encryption_key_ids(self):
        """Gets the encryption_key_ids of this SymmetricKeyUseDomain.  # noqa: E501

        these are the semmetric key id's that belong to this use domain - these keys never belong to any other use domain  # noqa: E501

        :return: The encryption_key_ids of this SymmetricKeyUseDomain.  # noqa: E501
        :rtype: list[str]
        """
        return self._encryption_key_ids

    @encryption_key_ids.setter
    def encryption_key_ids(self, encryption_key_ids):
        """Sets the encryption_key_ids of this SymmetricKeyUseDomain.

        these are the semmetric key id's that belong to this use domain - these keys never belong to any other use domain  # noqa: E501

        :param encryption_key_ids: The encryption_key_ids of this SymmetricKeyUseDomain.  # noqa: E501
        :type: list[str]
        """
        if encryption_key_ids is None:
            raise ValueError("Invalid value for `encryption_key_ids`, must not be `None`")  # noqa: E501

        self._encryption_key_ids = encryption_key_ids

    @property
    def endable_kds_fallback_to_cloud(self):
        """Gets the endable_kds_fallback_to_cloud of this SymmetricKeyUseDomain.  # noqa: E501

        if all registered kds service become unreachable, then incoming requests for new and existing keys may fallback to the cloud provided KDS  # noqa: E501

        :return: The endable_kds_fallback_to_cloud of this SymmetricKeyUseDomain.  # noqa: E501
        :rtype: bool
        """
        return self._endable_kds_fallback_to_cloud

    @endable_kds_fallback_to_cloud.setter
    def endable_kds_fallback_to_cloud(self, endable_kds_fallback_to_cloud):
        """Sets the endable_kds_fallback_to_cloud of this SymmetricKeyUseDomain.

        if all registered kds service become unreachable, then incoming requests for new and existing keys may fallback to the cloud provided KDS  # noqa: E501

        :param endable_kds_fallback_to_cloud: The endable_kds_fallback_to_cloud of this SymmetricKeyUseDomain.  # noqa: E501
        :type: bool
        """
        if endable_kds_fallback_to_cloud is None:
            raise ValueError("Invalid value for `endable_kds_fallback_to_cloud`, must not be `None`")  # noqa: E501

        self._endable_kds_fallback_to_cloud = endable_kds_fallback_to_cloud

    @property
    def require_signed_key_delivery(self):
        """Gets the require_signed_key_delivery of this SymmetricKeyUseDomain.  # noqa: E501

        if required, all clients must receive these keys in a signed symmetric key delivery from the key deriver  # noqa: E501

        :return: The require_signed_key_delivery of this SymmetricKeyUseDomain.  # noqa: E501
        :rtype: bool
        """
        return self._require_signed_key_delivery

    @require_signed_key_delivery.setter
    def require_signed_key_delivery(self, require_signed_key_delivery):
        """Sets the require_signed_key_delivery of this SymmetricKeyUseDomain.

        if required, all clients must receive these keys in a signed symmetric key delivery from the key deriver  # noqa: E501

        :param require_signed_key_delivery: The require_signed_key_delivery of this SymmetricKeyUseDomain.  # noqa: E501
        :type: bool
        """
        if require_signed_key_delivery is None:
            raise ValueError("Invalid value for `require_signed_key_delivery`, must not be `None`")  # noqa: E501

        self._require_signed_key_delivery = require_signed_key_delivery

    @property
    def digest_algorithm(self):
        """Gets the digest_algorithm of this SymmetricKeyUseDomain.  # noqa: E501

        The digest algorithm to use for signing messages in this use domain  # noqa: E501

        :return: The digest_algorithm of this SymmetricKeyUseDomain.  # noqa: E501
        :rtype: str
        """
        return self._digest_algorithm

    @digest_algorithm.setter
    def digest_algorithm(self, digest_algorithm):
        """Sets the digest_algorithm of this SymmetricKeyUseDomain.

        The digest algorithm to use for signing messages in this use domain  # noqa: E501

        :param digest_algorithm: The digest_algorithm of this SymmetricKeyUseDomain.  # noqa: E501
        :type: str
        """

        self._digest_algorithm = digest_algorithm

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(SymmetricKeyUseDomain, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SymmetricKeyUseDomain):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import pytest
from peacemakr.generated.api_client import ApiClient
from peacemakr.generated.configuration import Configuration
from peacemakr.generated.api.org_api import OrgApi
import peacemakr.factory as Factory
from peacemakr.impl.persister_impl import InMemoryPersister

import os
import random
import time
import string
from base64 import b64encode

# set up
@pytest.fixture
def setup_params():
    
    # set up test address
    test_url = os.getenv("PEACEMAKR_TEST_URL", "http://localhost:8080")

    # need to wait until server is setup, otherwise call to server will fail
    time.sleep(5)

    # set up test api_key
    configuration = Configuration()
    configuration.api_key['authorization'] = ""
    configuration.host = test_url + "/api/v1"
    api_client = ApiClient(configuration=configuration)

    org_api = OrgApi(api_client=api_client)
    api_key = None
    api_key = org_api.get_test_organization_api_key()
    assert api_key!=None, "No testing api key was found"

    params = {}
    params["api_key"] = api_key.key
    params["test_url"] = test_url
    return params


def test_register(setup_params):
    persister = InMemoryPersister()
    sdk = Factory.get_crypto_sdk(api_key=setup_params["api_key"], client_name="test_register", peacemakr_hostname=setup_params["test_url"], persister=persister)
    sdk.register()

    num_clients = persister.key_nums()
    assert(num_clients != 0)

def test_sync(setup_params):
    persister = InMemoryPersister()
    sdk = Factory.get_crypto_sdk(api_key=setup_params["api_key"], client_name="test_sync", peacemakr_hostname=setup_params["test_url"], persister=persister)
    sdk.register()

    # sleep for 1 sec to wait for sdk to have at least few keys ready
    # we should check for raise exception
    time.sleep(1)
    sdk.sync()

    num_clients = persister.key_nums()
    assert(num_clients != 0)

def test_encrypt_decrypt_bytes(setup_params):
    persister = InMemoryPersister()
    sdk = Factory.get_crypto_sdk(api_key=setup_params["api_key"], client_name="test_encrypt_decrypt_bytes", peacemakr_hostname=setup_params["test_url"], persister=persister)
    sdk.register()

    # sleep to wait for keys to be generated in server
    time.sleep(2)
    sdk.sync()

    num_clients = persister.key_nums()
    assert(num_clients != 0)

    # generate random bytes
    random_bytes = os.urandom(100)
    # encode random bytes to utf-8
    plain_text = random_bytes

    encrypted_text = sdk.encrypt(plain_text)
    assert(plain_text != encrypted_text)

    # ouptut is bytes
    decrypted_text = sdk.decrypt(encrypted_text)
    assert(decrypted_text == plain_text)

def test_encrypt_decrypt_string(setup_params):
    persister = InMemoryPersister()
    sdk = Factory.get_crypto_sdk(api_key=setup_params["api_key"], client_name="test_encrypt_decrypt_string", peacemakr_hostname=setup_params["test_url"], persister=persister)
    sdk.register()

    # sleep to wait for keys to be generated in server
    time.sleep(2)
    sdk.sync()

    num_clients = persister.key_nums()
    assert(num_clients != 0)

    N = random.randint(0,1000)
    plain_text = ''.join(random.choices(string.printable, k=N))

    encrypted_text = sdk.encrypt(plain_text.encode())
    assert(plain_text != encrypted_text)

    # ouptut is bytes
    decrypted_text = sdk.decrypt(encrypted_text)
    assert(decrypted_text == plain_text.encode())



# coding: utf-8

"""
    Peacemakr

    This API describes the Peacemakr services, which enable seamless application layer encryption and verification.  # noqa: E501

    OpenAPI spec version: 1.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import unittest

import peacemakr
from peacemakr.generated.api.crypto_config_api import CryptoConfigApi  # noqa: E501
from peacemakr.generated.rest import ApiException


class TestCryptoConfigApi(unittest.TestCase):
    """CryptoConfigApi unit test stubs"""

    def setUp(self):
        self.api = peacemakr.generated.api.crypto_config_api.CryptoConfigApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_add_existing_use_domain(self):
        """Test case for add_existing_use_domain

        Add an existing use domain to another crypto config.  # noqa: E501
        """
        pass

    def test_add_use_domain(self):
        """Test case for add_use_domain

        Add a new active use domain and attached it to the crypto config.  # noqa: E501
        """
        pass

    def test_get_crypto_config(self):
        """Test case for get_crypto_config

        Get the crypto configurations  # noqa: E501
        """
        pass

    def test_rapid_rotation_use_domain(self):
        """Test case for rapid_rotation_use_domain

        Rapid expiration of existing use doamin and immediately replacment with an identical use domain containing fresh keys  # noqa: E501
        """
        pass

    def test_remove_use_domain(self):
        """Test case for remove_use_domain

        Delete a fully expired use domain  # noqa: E501
        """
        pass

    def test_update_crypto_config(self):
        """Test case for update_crypto_config

        Update the crypto configuration, ONLY the clientKeyType clientKeyBitlength, and clientKeyTTL fields.  # noqa: E501
        """
        pass

    def test_update_crypto_config_fallback_to_cloud(self):
        """Test case for update_crypto_config_fallback_to_cloud

        Update an existing crypto config's asymmetricKeyTTL  # noqa: E501
        """
        pass

    def test_update_crypto_config_selector_scheme(self):
        """Test case for update_crypto_config_selector_scheme

        Update an existing crypto config's domainSelectorScheme  # noqa: E501
        """
        pass

    def test_update_expire_use_domain(self):
        """Test case for update_expire_use_domain

        Chnage expiration of a use domain  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Peacemakr

    This API describes the Peacemakr services, which enable seamless application layer encryption and verification.  # noqa: E501

    OpenAPI spec version: 1.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from peacemakr.generated.api_client import ApiClient


class KeyServiceApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def get_all_encrypted_keys(self, encrypting_key_id, **kwargs):  # noqa: E501
        """Get all encrypted symmetric keys that are encrypted with this encrypting keyId, optionally limiting the request to a set of symmetric key domains  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_all_encrypted_keys(encrypting_key_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str encrypting_key_id: (required)
        :param list[str] symmetric_key_ids:
        :return: list[EncryptedSymmetricKey]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_all_encrypted_keys_with_http_info(encrypting_key_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_all_encrypted_keys_with_http_info(encrypting_key_id, **kwargs)  # noqa: E501
            return data

    def get_all_encrypted_keys_with_http_info(self, encrypting_key_id, **kwargs):  # noqa: E501
        """Get all encrypted symmetric keys that are encrypted with this encrypting keyId, optionally limiting the request to a set of symmetric key domains  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_all_encrypted_keys_with_http_info(encrypting_key_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str encrypting_key_id: (required)
        :param list[str] symmetric_key_ids:
        :return: list[EncryptedSymmetricKey]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['encrypting_key_id', 'symmetric_key_ids']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_all_encrypted_keys" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'encrypting_key_id' is set
        if ('encrypting_key_id' not in params or
                params['encrypting_key_id'] is None):
            raise ValueError("Missing the required parameter `encrypting_key_id` when calling `get_all_encrypted_keys`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'encrypting_key_id' in params:
            path_params['encryptingKeyId'] = params['encrypting_key_id']  # noqa: E501

        query_params = []
        if 'symmetric_key_ids' in params:
            query_params.append(('symmetricKeyIds', params['symmetric_key_ids']))  # noqa: E501
            collection_formats['symmetricKeyIds'] = 'csv'  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['header']  # noqa: E501

        return self.api_client.call_api(
            '/crypto/symmetric/{encryptingKeyId}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[EncryptedSymmetricKey]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_public_key(self, key_id, **kwargs):  # noqa: E501
        """Get the public key associated with the passed-in key ID  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_public_key(key_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str key_id: (required)
        :return: PublicKey
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_public_key_with_http_info(key_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_public_key_with_http_info(key_id, **kwargs)  # noqa: E501
            return data

    def get_public_key_with_http_info(self, key_id, **kwargs):  # noqa: E501
        """Get the public key associated with the passed-in key ID  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_public_key_with_http_info(key_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str key_id: (required)
        :return: PublicKey
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['key_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_public_key" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'key_id' is set
        if ('key_id' not in params or
                params['key_id'] is None):
            raise ValueError("Missing the required parameter `key_id` when calling `get_public_key`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'key_id' in params:
            path_params['keyID'] = params['key_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['header']  # noqa: E501

        return self.api_client.call_api(
            '/crypto/asymmetric/{keyID}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='PublicKey',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def post_new_encrypted_keys(self, encrypting_key_id, encrypted_symmetric_key, **kwargs):  # noqa: E501
        """Add a new encrypted key. The encrypting key that protects the encrypted key is identified with encryptingKeyId. Request must come from a registered key manager.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_new_encrypted_keys(encrypting_key_id, encrypted_symmetric_key, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str encrypting_key_id: (required)
        :param list[EncryptedSymmetricKey] encrypted_symmetric_key: (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.post_new_encrypted_keys_with_http_info(encrypting_key_id, encrypted_symmetric_key, **kwargs)  # noqa: E501
        else:
            (data) = self.post_new_encrypted_keys_with_http_info(encrypting_key_id, encrypted_symmetric_key, **kwargs)  # noqa: E501
            return data

    def post_new_encrypted_keys_with_http_info(self, encrypting_key_id, encrypted_symmetric_key, **kwargs):  # noqa: E501
        """Add a new encrypted key. The encrypting key that protects the encrypted key is identified with encryptingKeyId. Request must come from a registered key manager.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_new_encrypted_keys_with_http_info(encrypting_key_id, encrypted_symmetric_key, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str encrypting_key_id: (required)
        :param list[EncryptedSymmetricKey] encrypted_symmetric_key: (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['encrypting_key_id', 'encrypted_symmetric_key']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method post_new_encrypted_keys" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'encrypting_key_id' is set
        if ('encrypting_key_id' not in params or
                params['encrypting_key_id'] is None):
            raise ValueError("Missing the required parameter `encrypting_key_id` when calling `post_new_encrypted_keys`")  # noqa: E501
        # verify the required parameter 'encrypted_symmetric_key' is set
        if ('encrypted_symmetric_key' not in params or
                params['encrypted_symmetric_key'] is None):
            raise ValueError("Missing the required parameter `encrypted_symmetric_key` when calling `post_new_encrypted_keys`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'encrypting_key_id' in params:
            path_params['encryptingKeyId'] = params['encrypting_key_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'encrypted_symmetric_key' in params:
            body_params = params['encrypted_symmetric_key']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['header']  # noqa: E501

        return self.api_client.call_api(
            '/crypto/symmetric/{encryptingKeyId}', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



# coding: utf-8

"""
    Peacemakr

    This API describes the Peacemakr services, which enable seamless application layer encryption and verification.  # noqa: E501

    OpenAPI spec version: 1.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import unittest

import peacemakr
from peacemakr.generated.api.client_api import ClientApi  # noqa: E501
from peacemakr.generated.rest import ApiException


class TestClientApi(unittest.TestCase):
    """ClientApi unit test stubs"""

    def setUp(self):
        self.api = peacemakr.generated.api.client_api.ClientApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_add_client(self):
        """Test case for add_client

        Register a new client  # noqa: E501
        """
        pass

    def test_add_client_public_key(self):
        """Test case for add_client_public_key

        Register a new public key for the client  # noqa: E501
        """
        pass

    def test_delete_client(self):
        """Test case for delete_client

        Remove an existing organization  # noqa: E501
        """
        pass

    def test_get_client(self):
        """Test case for get_client

        Get an existing client  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Peacemakr

    This API describes the Peacemakr services, which enable seamless application layer encryption and verification.  # noqa: E501

    OpenAPI spec version: 1.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from peacemakr.generated.api_client import ApiClient


class ClientApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def add_client(self, client, **kwargs):  # noqa: E501
        """Register a new client  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_client(client, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param Client client: (required)
        :return: Client
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.add_client_with_http_info(client, **kwargs)  # noqa: E501
        else:
            (data) = self.add_client_with_http_info(client, **kwargs)  # noqa: E501
            return data

    def add_client_with_http_info(self, client, **kwargs):  # noqa: E501
        """Register a new client  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_client_with_http_info(client, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param Client client: (required)
        :return: Client
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['client']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method add_client" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'client' is set
        if ('client' not in params or
                params['client'] is None):
            raise ValueError("Missing the required parameter `client` when calling `add_client`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'client' in params:
            body_params = params['client']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['header']  # noqa: E501

        return self.api_client.call_api(
            '/client', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Client',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def add_client_public_key(self, client_id, new_public_key, **kwargs):  # noqa: E501
        """Register a new public key for the client  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_client_public_key(client_id, new_public_key, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str client_id: (required)
        :param PublicKey new_public_key: (required)
        :return: PublicKey
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.add_client_public_key_with_http_info(client_id, new_public_key, **kwargs)  # noqa: E501
        else:
            (data) = self.add_client_public_key_with_http_info(client_id, new_public_key, **kwargs)  # noqa: E501
            return data

    def add_client_public_key_with_http_info(self, client_id, new_public_key, **kwargs):  # noqa: E501
        """Register a new public key for the client  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_client_public_key_with_http_info(client_id, new_public_key, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str client_id: (required)
        :param PublicKey new_public_key: (required)
        :return: PublicKey
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['client_id', 'new_public_key']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method add_client_public_key" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'client_id' is set
        if ('client_id' not in params or
                params['client_id'] is None):
            raise ValueError("Missing the required parameter `client_id` when calling `add_client_public_key`")  # noqa: E501
        # verify the required parameter 'new_public_key' is set
        if ('new_public_key' not in params or
                params['new_public_key'] is None):
            raise ValueError("Missing the required parameter `new_public_key` when calling `add_client_public_key`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'client_id' in params:
            path_params['clientId'] = params['client_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'new_public_key' in params:
            body_params = params['new_public_key']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['header']  # noqa: E501

        return self.api_client.call_api(
            '/client/{clientId}/addPublicKey', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='PublicKey',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def delete_client(self, client_id, **kwargs):  # noqa: E501
        """Remove an existing organization  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_client(client_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str client_id: (required)
        :return: Client
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.delete_client_with_http_info(client_id, **kwargs)  # noqa: E501
        else:
            (data) = self.delete_client_with_http_info(client_id, **kwargs)  # noqa: E501
            return data

    def delete_client_with_http_info(self, client_id, **kwargs):  # noqa: E501
        """Remove an existing organization  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_client_with_http_info(client_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str client_id: (required)
        :return: Client
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['client_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_client" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'client_id' is set
        if ('client_id' not in params or
                params['client_id'] is None):
            raise ValueError("Missing the required parameter `client_id` when calling `delete_client`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'client_id' in params:
            path_params['clientId'] = params['client_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['header']  # noqa: E501

        return self.api_client.call_api(
            '/client/{clientId}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Client',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_client(self, client_id, **kwargs):  # noqa: E501
        """Get an existing client  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_client(client_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str client_id: (required)
        :return: Client
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_client_with_http_info(client_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_client_with_http_info(client_id, **kwargs)  # noqa: E501
            return data

    def get_client_with_http_info(self, client_id, **kwargs):  # noqa: E501
        """Get an existing client  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_client_with_http_info(client_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str client_id: (required)
        :return: Client
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['client_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_client" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'client_id' is set
        if ('client_id' not in params or
                params['client_id'] is None):
            raise ValueError("Missing the required parameter `client_id` when calling `get_client`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'client_id' in params:
            path_params['clientId'] = params['client_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['header']  # noqa: E501

        return self.api_client.call_api(
            '/client/{clientId}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Client',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)
